import BaseFormComponent, { BaseConfig } from "./entity/base";

/**
 * 表示一个抽象类型
 */
export interface Abstract<T extends object = object> extends Function {
  /** 实例原型 */
  readonly prototype: T;
}

/**
 * 表示一个可构造类型
 */
export interface Constructable<T extends object = object> extends Abstract<T> {
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  new (...args: any[]): T;
}

/**
 * 表示一个 class
 */
export type Class<T extends object = object> = Abstract<T> | Constructable<T>;

/**
 * 基础类型的表示
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
export type Primitive<T = any> = T extends string
  ? typeof String
  : T extends number
  ? typeof Number
  : T extends boolean
  ? typeof Boolean
  : T extends symbol
  ? typeof Symbol // : T extends bigint ? typeof BigInt
  : never;

/**
 * 表示一个类型
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
export type Type<T = any> = T extends object ? Class<T> : Primitive<T>;

export interface FormComponentRegistryInfo {
  icon?: string;
  description?: string;
  name: string;
  componentType?: string;
}

/**
 * 表示字段元数据的 map
 */
export type FieldInfoMap<U, T> = Record<keyof T, U>;

export interface FormConfig<T extends BaseConfig = BaseConfig> {
  group: string;
  label: string;
  componentType: string;
  key: string;
  componentOption: T;
  config?: Partial<{
    validationRules: {
      require: boolean;
    };
    print: boolean;
    templateUrl: string;
  }>;
}

/** 元数据属性值 */
export interface PropInfo {
  def?: any;
  /** 属性的描述 */
  desc?: string;
  /** 属性值的特殊类型 */
  type?: ComponentsType;
  /** 属性相关配置 */
  config?: Record<string, any>;
}

/**
 * 属性元数据 Map
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
export type PropInfoMap<T = any> = FieldInfoMap<PropInfo, T>;

export interface ObjectMetadata<
  T extends BaseFormComponent = BaseFormComponent
> {
  /** 类名 */
  name?: string;
  /** 类型 */
  type: Type<T>;
  /**
   * 属性定义
   */
  properties: PropInfoMap<T>;
  /**
   * 组件元数据
   */
  component: FormComponentRegistryInfo;
}

export interface ObjectConfigMetadata<T extends BaseConfig = BaseConfig> {
  /**
   * 属性定义
   */
  properties: PropInfoMap<T>;
}

export type KeyOf<T extends BaseFormComponent> =
  | Exclude<keyof T, keyof BaseFormComponent>
  | "key";

export type ComponentsType =
  | "Radio"
  | "FileUpload"
  | "Branch"
  | "DatePicker"
  | "ImgUpload"
  | "NumberInput"
  | "PhoneInput"
  | "Select"
  | "TextInput"
  | "TextAreaInput"
  | "Checkbox"
  | "List"
  | "UserSelect";

type PropertyNames<T> = {
  [K in keyof T]: T[K] extends Function ? never : K;
}[keyof T];

export type OnlyProperties<T> = Pick<T, PropertyNames<T>>;
