import BasicModal from "@/components/BasicModal"
import { FC, useEffect, useRef, useState } from "react"
import styles from './ApproveVisual.module.css'
import modalIcon from "@/assets/images/visual/modalIcon.png";
import headIcon from "@/assets/images/visual/headIcon.png";
import icon1 from "@/assets/images/visual/icon1.png";
import icon2 from "@/assets/images/visual/icon2.png";
import { Button, Flex, Table, TableProps } from "antd";
import PieChart from "./components/PieChart/PieChart";
import LineChart from "./components/LineChart/LineChart";
import EllipsisBox from "@/components/EllipsisBox/EllipsisBox";
import { getStatisc, rankList, taskAnalyseExportExcel, taskAnalysePageList } from "@/service/approveVisual";
import { useSetState } from "ahooks";
import avatarIcon from "@/assets/images/user/avatar.png";
import { downFlowFile, exportPDF } from "@/utils";


type ApproveVisualProps = {
    visible: boolean
    onClose: () => void
}

const ApproveVisual: FC<ApproveVisualProps> = (props) => {
    const { visible, onClose } = props

    const columns: TableProps<any>["columns"] = [
        { title: '任务类型', dataIndex: 'name' },
        { title: '审批完成率', align: 'center', width: 110, dataIndex: 'completionRate', render: (text) => text + '%' },
        { title: '平均处置时长', align: 'center', width: 120, dataIndex: 'averageDuration' },
        { title: '最短处置时长', align: 'center', width: 120, dataIndex: 'shortestLength' },
        { title: '最长处置时长', align: 'center', width: 120, dataIndex: 'longestLength' },
    ]

    const [statiscData, setStatiscData] = useState({ allStatisc: [] as any[], statusStatisc: [] as any[] })
    const getStatiscData = async () => {
        const res = await getStatisc()
        setStatiscData(res)
    }

    const [pageQuery, setPageQuery] = useSetState({ page: 1, count: 10 })
    const [list, setList] = useState<any[]>([])
    const [total, setTotal] = useState(0)
    const getTaskAnalyse = async () => {
        const res = await taskAnalysePageList(pageQuery)
        if (res.code === 200) {
            setTotal(res.data.total)
            setList(res.data.items)
        }
    }

    const [rankDatas, setRankDatas] = useState<any[]>([])
    const getRanks = async () => {
        const res = await rankList()
        if (res.code === 200) {
            const data = res.data.map((v: any, i: number) => {
                if (i === 0) v.color = '#FBC55C'
                if (i === 1) v.color = '#80DA80'
                if (i === 2) v.color = '#9DA1EF'
                return v
            })
            setRankDatas(data)
        }
    }

    useEffect(() => {
        if (visible) {
            getTaskAnalyse()
        }
    }, [visible, pageQuery])

    useEffect(() => {
        if (visible) {
            getStatiscData()
            getRanks()
        }
    }, [visible])

    const exportExcel = async () => {
        const res = await taskAnalyseExportExcel()
        downFlowFile(res, '任务分析数据.xlsx')
    }

    const boxRef = useRef<HTMLElement>(null)
    const exportPdf = () => {
        if (boxRef.current) {
            exportPDF('审批通可视化', boxRef.current)
        }
    }

    return <BasicModal
        visible={visible}
        onClose={onClose}
        width={1600}
        contentStyle={{ borderRadius: 10, overflow: 'hidden' }}
        height={1000}
    >
        <Flex className={styles.container} vertical ref={boxRef}>
            <div className={styles.header}>
                <img src={modalIcon} style={{ width: 46, height: 46 }} />
                <div className={styles.headerTitle}>审批通可视化</div>
            </div>
            <Flex className={styles.contentBox} flex={1} gap={35}>
                <Flex className={styles.left} vertical gap={20}>
                    <Flex className={styles.top1} align="center" justify='space-between'>
                        <Flex gap={10} align="center">
                            <img width={90} src={headIcon} />
                            <Flex vertical gap={10} align="center">
                                <div className={styles.txt}>全部</div>
                                <div className={styles.num}>{statiscData.allStatisc[0]?.value}</div>
                            </Flex>
                        </Flex>
                        <Flex vertical gap={12}>
                            <Flex align="center" gap={10}>
                                <img width={15} src={icon1} />
                                <span className={`${styles.txt} ${styles.fs16}`}>行政备案</span>
                                <div className={styles.num} style={{ fontSize: 22, marginLeft: 5 }}>{statiscData.allStatisc[1]?.value}</div>
                            </Flex>
                            <Flex align="center" gap={10}>
                                <img width={15} src={icon2} />
                                <span className={`${styles.txt} ${styles.fs16}`}>行政许可</span>
                                <div className={styles.num} style={{ fontSize: 22, marginLeft: 5 }}>{statiscData.allStatisc[2]?.value}</div>
                            </Flex>
                        </Flex>
                    </Flex>
                    <div className={styles.top1} style={{ height: 105, padding: '25px 40px', display: 'flex', justifyContent: 'space-between' }}>
                        {statiscData.statusStatisc.map((v, i) => {
                            const { name, value, color } = v
                            return <Flex className={styles.item} vertical justify='space-between' key={i}>
                                <Flex className={`${styles.txt} ${styles.fs16}`} align="center" gap={5}>
                                    <i style={{ background: color }} /> {name}
                                </Flex>
                                <div className={styles.num} style={{ fontSize: 22, marginLeft: 10 }}>{value}</div>
                            </Flex>
                        })}
                    </div>
                    <PieChart style={{ width: '100%', height: 200 }} />
                    <LineChart style={{ width: '100%', flex: 1 }} />
                </Flex>
                <Flex className={styles.left} flex={1} vertical gap={25} style={{ background: 'transparent', padding: 0 }}>
                    <Flex className={styles.tableBox} vertical gap={30}>
                        <Flex align="center" justify='space-between'>
                            <div className={styles.title}>任务分析</div>
                            <Flex gap={10}>
                                <Button type="primary" onClick={exportPdf}>导出PDF</Button>
                                <Button type="primary" onClick={exportExcel}>导出表格</Button>
                            </Flex>
                        </Flex>
                        <Table
                            rowKey='name'
                            scroll={{ y: 330 }}
                            columns={columns}
                            dataSource={list}
                            pagination={{
                                showTotal: (total) => `共 ${total} 条`,
                                total,
                                current: pageQuery.page,
                                pageSize: pageQuery.count,
                                showSizeChanger: true,
                                onShowSizeChange(_, pageSize) {
                                    setPageQuery({ count: pageSize, page: 1 })
                                },
                                onChange: (pageNum) => {
                                    setPageQuery({ page: pageNum })
                                },
                            }}
                        />
                    </Flex>
                    <Flex className={styles.bottom} flex={1} gap={25}>
                        <Flex className={styles.item} vertical gap={20}>
                            <div className={styles.title}>审批人员绩效分析</div>
                            <Flex className={styles.listBox} justify='space-between' flex={1}>
                                {rankDatas.map((v, i) => {
                                    const { id, averageDuration, completionRate, name, color } = v
                                    return <Flex className={styles.listItem} vertical justify='space-between' key={id}>
                                        <Flex className={styles.head} align="center">
                                            <img width={48} src={avatarIcon} />
                                            {name}
                                            <Flex className={styles.rank} style={{ background: color }} justify="center">{i + 1}</Flex>
                                        </Flex>
                                        <Flex className={styles.overRate}>
                                            <span>审批完成率：</span>
                                            <span><span className={styles.rateNum}>{completionRate}</span> %</span>
                                        </Flex>
                                        <Flex className={styles.overRate}>
                                            <span>平均处置时长：</span>
                                            <span><span className={styles.rateNum}>{averageDuration}</span> h</span>
                                        </Flex>
                                    </Flex>
                                })}
                            </Flex>
                        </Flex>
                        <Flex className={styles.item} vertical gap={20} flex={1}>
                            <div className={styles.title}>流程优化建议</div>
                            <Flex vertical flex={1} gap={10}>
                                <Flex className={styles.suggest} align="center">
                                    <EllipsisBox line={3} txt={'临时性建筑物搭建、堆放物料、占道施工审批平均处理时长最长，建议优化其流程'} />
                                </Flex>
                                <Flex className={styles.suggest} align="center">
                                    <EllipsisBox line={3} txt={'推行并联审批制度，对于一些不相互冲突的审批环节，允许同时进行审核，而不是按照传统的顺序依次进行。'} />
                                </Flex>
                            </Flex>
                        </Flex>
                    </Flex>
                </Flex>
            </Flex>
        </Flex>
    </BasicModal>
}

export default ApproveVisual