import { ConfigProvider, Flex, Input, Space, Table, TableProps } from "antd";
import styles from "./listTable.module.css";
import classNames from "classnames";
import { useEffect, useMemo, useState } from "react";
import { SearchOutlined } from "@ant-design/icons";

import ApplicationMaterial from "@/pages/ApplicationMaterial";
import { useBoolean, useThrottleEffect } from "ahooks";

import service from "@/service";
import eventBus from "@/utils/eventBus";

const TagTab = ({ activeTab, tags, onChange }: any) => {
  return (
    <Flex gap={14} align="center" wrap="wrap">
      {tags.map((item: any, index: number) => {
        return (
          <div
            key={index}
            onClick={onChange.bind(this, index)}
            className={classNames(
              styles.tagItem,
              index === activeTab ? styles.active : ""
            )}
          >
            {item.elementName}&nbsp;{item.elementCount}
          </div>
        );
      })}
    </Flex>
  );
};

const ListTable = (props: {
  applicationInfo: any;
  active?: string;
  type: string;
}) => {
  const { applicationInfo, active, type } = props;
  const [showDetail, { setFalse, setTrue }] = useBoolean(false);
  const [actionObj, setActionObj] = useState<any>();
  const [activeTab, setActiveTab] = useState(0);
  const [inputStr, setInputStr] = useState("");
  const [paginationInfo, setPaginationInfo] = useState<any>();
  const [dataSource, setDataSource] = useState<any[]>([]);
  const [modalOpt, setModalOpt] = useState<any>({});

  const flowList = useMemo(() => {
    console.log(type);
    return (
      applicationInfo?.nodeCount?.filter((v: any) => {
        if (type === "deal") {
          return v.elementId === "APPROVAL" || v.elementId === "APPROVED";
        }
        if (type === "manage") {
          return v.elementId !== "APPROVAL" && v.elementId !== "APPROVED";
        }
        return false;
      }) ?? []
    );
  }, [applicationInfo, type]);

  useEffect(() => {
    if (active && active.length > 0) {
      const index = flowList.findIndex(
        (item: any) => item.elementId === active
      );
      setActiveTab(index);
    } else {
      setActiveTab(0);
    }
  }, [active]);

  /** 请求时间过短，手动节流 */
  useThrottleEffect(
    () => {
      getData(1);
    },
    [activeTab, inputStr, applicationInfo],
    {
      wait: 300,
    }
  );

  useEffect(() => {
    const handler = () => {
      setFalse();
    };
    eventBus.on("updateTableList", handler);
    return () => eventBus.off("updateTableList", handler);
  }, []);

  const getData = (pageNum: number) => {
    const elementId = flowList[activeTab]?.elementId;
    let params: any = {
      thingId: applicationInfo.id,
      name: inputStr,
      elementId: elementId,
      page: pageNum,
      count: 6,
    };

    if (!elementId) {
      params = { ...params, showAll: true };
    }

    service.managementService.getApplicationRecordById(params).then((res) => {
      if (res.code === 200) {
        setDataSource(res.data?.items);
        setPaginationInfo(res.data);
      }
    });
  };

  const columns: TableProps<any>["columns"] = [
    {
      title: "申请编号",
      dataIndex: "id",
    },
    {
      title: "申请单位",
      dataIndex: "subjectName",
      key: "subjectName",
    },
    {
      title: "申请时间",
      dataIndex: "createAt",
      key: "createAt",
    },
    {
      title: "办理阶段",
      key: "nodeNameAlias",
      render(_, record) {
        return record.nodeNameAlias ?? record?.elementName;
      },
    },
    {
      title: "办理说明",
      dataIndex: "approveComment",
      key: "approveComment",
      render(_, record) {
        return record.nodeRemake ?? record.Remake ?? "无";
      },
    },
    {
      title: "操作",
      key: "action",
      render: (_, record) => {
        const elementId = flowList[activeTab]?.elementId;
        return (
          <div>
            {elementId === "APPROVAL" ? (
              <a
                onClick={() => {
                  setActionObj(record);
                  setTrue();
                  setModalOpt({
                    type: "edit",
                  });
                }}
              >
                办理
              </a>
            ) : (
              <a
                onClick={() => {
                  setActionObj(record);
                  setModalOpt({
                    type: "detail",
                  });
                  setTrue();
                }}
              >
                查看详情
              </a>
            )}
          </div>
        );
      },
    },
  ];

  const clickHandle = (index: number) => {
    setActiveTab(index);
    if (index !== activeTab) {
      setInputStr("");
    }
  };

  const inputChange = (e: any) => {
    const value = e.target.value;
    setInputStr(value);
  };

  return (
    <>
      <ApplicationMaterial
        visible={showDetail}
        mode="review"
        modalOpt={modalOpt}
        actionObj={actionObj}
        onClose={setFalse}
      ></ApplicationMaterial>
      <div className={styles.container}>
        <Flex
          style={{ margin: "24px 0" }}
          align="flex-start"
          justify="space-between"
        >
          <TagTab
            activeTab={activeTab}
            tags={flowList}
            onChange={clickHandle}
          ></TagTab>
          <Space>
            <Input
              placeholder="输入申请单位名称搜索"
              prefix={<SearchOutlined />}
              style={{ height: 50 }}
              value={inputStr}
              onChange={inputChange}
            ></Input>
            {/* <div className={styles.iconBtn}>
              <img
                src={listIcon}
                alt=""
                style={{ width: 18, height: 18 }}
              ></img>
            </div>
            <div
              className={styles.iconBtn}
              onClick={() => {
                message.info("功能待开发");
              }}
            >
              <img
                src={cardIcon}
                alt=""
                style={{ width: 18, height: 18 }}
              ></img>
            </div>
            <div
              className={styles.iconBtn}
              onClick={() => {
                message.info("功能待开发");
              }}
            >
              <img src={mapIcon} alt="" style={{ width: 18, height: 18 }}></img>
            </div> */}
          </Space>
        </Flex>
        <ConfigProvider
          theme={{
            components: {
              Table: {
                headerColor: "rgba(139, 146, 159, 1)",
                headerBg: "#fff",
                rowHoverBg: "#EDF5FF",
                borderColor: "transparent",
              },
            },
          }}
        >
          <Table
            columns={columns}
            dataSource={dataSource}
            pagination={{
              pageSize: 6,
              total: paginationInfo?.total,
              current: paginationInfo?.page,
              onChange: (pageNum) => {
                getData(pageNum);
              },
            }}
          />
        </ConfigProvider>
      </div>
    </>
  );
};

export default ListTable;
