import BasicModal from "@/components/BasicModal"
import { FC, useEffect, useRef, useState } from "react"
import styles from './SubjectDataVisual.module.css'
import modalIcon from "@/assets/images/visual/modalIcon.png";
import merchantIcon from "@/assets/images/visual/merchantIcon.png";
import { Flex, Select, Table, TableProps, } from "antd";
import PieChart from "./components/PieChart/PieChart";
import { mapMarkers, streetMerchants, streetOption, typeMerchantPageList } from "@/service/subjectDataVisual";
import mapStyle from '@/utils/mapStyle_map_config.json'
import { CustomOverlay, Map } from "react-bmapgl"
import { useSetState } from "ahooks";
import EllipsisBox from "@/components/EllipsisBox/EllipsisBox";

type SubjectDataVisualProps = {
    visible: boolean
    onClose: () => void
}

const SubjectDataVisual: FC<SubjectDataVisualProps> = (props) => {
    const { visible, onClose } = props
    const map = useRef<BMapGL.Map>()

    const columns: TableProps<any>["columns"] = [
        { title: '事项', dataIndex: 'matterName' },
        { title: '数量', width: 100, align: 'center', dataIndex: 'totalNum' },
        { title: '商户数', width: 100, align: 'center', dataIndex: 'merchantNum' }
    ]

    const [pageQuery, setPageQuery] = useSetState({ page: 1, count: 10 })
    const [list, setList] = useState<any[]>([])
    const [total, setTotal] = useState(0)
    const getTypeMerchant = async () => {
        const res = await typeMerchantPageList(pageQuery)
        if (res.code === 200) {
            setTotal(res.data.total)
            setList(res.data.items)
        }
    }

    useEffect(() => {
        if (visible) {
            getTypeMerchant()
            getAreaDatas()
            getstreetOption()
        }
    }, [visible])

    const [areaDatas, setAreaDatas] = useState<any[]>([])
    const getAreaDatas = async () => {
        const res = await streetMerchants()
        if (res.code === 200) {
            setAreaDatas(res.data)
        }
    }

    const [streetCode, setStreetCode] = useState('')
    const [streetOptions, setStreetOptions] = useState<any[]>([])
    const getstreetOption = async () => {
        const res = await streetOption()
        if (res.code === 200) {
            setStreetOptions(res.data)
        }
    }

    const [markers, setMarkers] = useState<any[]>([])
    const getMarkers = async () => {
        const res = await mapMarkers(streetCode)
        if (res.code === 200) {
            setMarkers(res.data)
            const points = res.data.map((v: any) => {
                const { lat, lon } = v
                return new BMapGL.Point(Number(lon), Number(lat))
            })
            map.current?.setViewport(points)
        }
    }
    useEffect(() => {
        if (visible) {
            getMarkers()
        }
    }, [visible, streetCode])

    return <BasicModal
        visible={visible}
        onClose={onClose}
        width={1600}
        contentStyle={{ borderRadius: 10, overflow: 'hidden' }}
        height={1000}
    >
        <Flex className={styles.container} vertical>
            <div className={styles.header}>
                <img src={modalIcon} style={{ width: 46, height: 46 }} />
                <div className={styles.headerTitle}>主体数据可视化</div>
            </div>
            <Flex className={styles.contentBox} flex={1} gap={35}>
                <Flex className={styles.left} vertical gap={20}>
                    <Flex className={styles.top1} vertical gap={10} flex={1}>
                        <div className={styles.title}>任务类型商户</div>
                        <Table
                            rowKey='matterName'
                            scroll={{ y: 200 }}
                            columns={columns}
                            dataSource={list}
                            pagination={{
                                showTotal: (total) => `共 ${total} 条`,
                                total,
                                current: pageQuery.page,
                                pageSize: pageQuery.count,
                                showSizeChanger: true,
                                onShowSizeChange(_, pageSize) {
                                    setPageQuery({ count: pageSize, page: 1 })
                                },
                                onChange: (pageNum) => {
                                    setPageQuery({ page: pageNum })
                                },
                            }}
                        />
                    </Flex>
                    <PieChart style={{ width: '100%', flex: 1 }} />
                    <Flex className={styles.top1} style={{ height: 300 }} vertical gap={10}>
                        <div className={styles.title}>区域</div>
                        <Flex className={styles.titleBox}>
                            <div>街道</div>
                            <div>一般商户数</div>
                            <div>外包商户数</div>
                        </Flex>
                        <Flex className={styles.listBox} vertical gap={5} flex={1}>
                            {areaDatas.map((v, i) => {
                                const { ordinaryMerchants, outsourcing, street } = v
                                return <Flex className={styles.listItem} key={i}>
                                    <div>{street}</div>
                                    <div>{ordinaryMerchants}</div>
                                    <div>{outsourcing}</div>
                                </Flex>
                            })}
                        </Flex>
                    </Flex>
                </Flex>
                <div className={styles.mapBox}>
                    <Flex className={styles.filterBox}>
                        {/* <Input className={styles.inp} placeholder="按区域搜所" onKeyUp={(e) => {
                            if (e.key === 'Enter') {
                                const val = e.currentTarget.value
                                console.log(val);
                            }
                        }} /> */}
                        <Select
                            variant='borderless'
                            className={styles.inp}
                            placeholder='按区域搜所'
                            showSearch
                            options={streetOptions}
                            popupMatchSelectWidth={false}
                            optionFilterProp="label"
                            onChange={(val) => { setStreetCode(val) }}
                        />
                    </Flex>

                    {/* @ts-ignore */}
                    <Map
                        mapStyleV2={{ styleJson: mapStyle }}
                        style={{ height: '100%' }}
                        ref={(data) => map.current = data?.map}
                        center={new BMapGL.Point(104.1756642, 30.8230263)}
                        zoom={16}
                        enableScrollWheelZoom
                    >
                        {markers.length !== 0 && <>
                            {markers.map((v) => {
                                const { id, lat, lon, name } = v
                                const point = new BMapGL.Point(Number(lon), Number(lat))

                                {/* @ts-ignore */ }
                                return <CustomOverlay
                                    map={map.current as BMapGL.Map}
                                    key={id}
                                    position={point}
                                >
                                    <Flex vertical justify="center" align="center" gap={5}>
                                        <div className={styles.mapTxt}>
                                            <EllipsisBox txt={name} />
                                        </div>
                                        <img width={46} height={52} src={merchantIcon} />
                                    </Flex>
                                </CustomOverlay>
                            })}
                        </>}
                    </Map>
                </div>
            </Flex>
        </Flex>
    </BasicModal>
}

export default SubjectDataVisual