import BaseFormComponent from "@/customForm/entity/base";
import { Button, ConfigProvider, Modal, Popover, Space, Table } from "antd";
import { ColumnsType } from "antd/es/table";
import { useContext, useMemo, useState } from "react";
import {
  DeleteOutlined,
  HolderOutlined,
  PlusCircleOutlined,
} from "@ant-design/icons";
import type { DragEndEvent } from "@dnd-kit/core";
import { DndContext } from "@dnd-kit/core";
import type { SyntheticListenerMap } from "@dnd-kit/core/dist/hooks/utilities";
import { restrictToVerticalAxis } from "@dnd-kit/modifiers";
import {
  arrayMove,
  SortableContext,
  useSortable,
  verticalListSortingStrategy,
} from "@dnd-kit/sortable";
import { CSS } from "@dnd-kit/utilities";
import React from "react";
import styles from "./complex-table.module.css";
import Menu from "./menu";

import Checkbox from "@/customForm/components/Checkbox";
import { useFormContext } from "..";
import { buildTreeByGroup } from "@/customForm/utils";
import { cloneDeep } from "lodash";

interface RowContextProps {
  setActivatorNodeRef?: (element: HTMLElement | null) => void;
  listeners?: SyntheticListenerMap;
}

const RowContext = React.createContext<RowContextProps>({});

const DragHandle: React.FC = () => {
  const { setActivatorNodeRef, listeners } = useContext(RowContext);
  return (
    <Button
      type="text"
      size="small"
      icon={<HolderOutlined />}
      style={{ cursor: "move" }}
      ref={setActivatorNodeRef}
      {...listeners}
    />
  );
};

interface RowProps extends React.HTMLAttributes<HTMLTableRowElement> {
  "data-row-key": string;
}

const Row: React.FC<RowProps> = (props) => {
  const {
    attributes,
    listeners,
    setNodeRef,
    setActivatorNodeRef,
    transform,
    transition,
    isDragging,
  } = useSortable({ id: props["data-row-key"] });

  const style: React.CSSProperties = {
    ...props.style,
    transform: CSS.Translate.toString(transform),
    transition,
    ...(isDragging ? { position: "relative", zIndex: 9999 } : {}),
  };

  const contextValue = useMemo<RowContextProps>(
    () => ({ setActivatorNodeRef, listeners }),
    [setActivatorNodeRef, listeners]
  );

  return (
    <RowContext.Provider value={contextValue}>
      <tr {...props} ref={setNodeRef} style={style} {...attributes} />
    </RowContext.Provider>
  );
};
interface ComplexTableProps {
  onClickRow: (row: BaseFormComponent) => {
    onClick: () => void;
  };
  activeRow?: BaseFormComponent;
}

const ComplexTable = ({ onClickRow, activeRow }: ComplexTableProps) => {
  const { setFormConfigList, formConfigList } = useFormContext();

  const [popoverOpen, setPopoverOpen] = useState<Record<string, boolean>>({});

  const TableData = useMemo(() => {
    const branch = formConfigList.filter(
      (item) => item.componentType === "Branch"
    );
    if (branch.length > 0) {
      setPopoverOpen(
        branch.reduce((pre, cur) => {
          return { ...pre, [cur.key]: false };
        }, {})
      );
    }
    return buildTreeByGroup(formConfigList);
  }, [formConfigList]);
  /** 删除节点 */
  const deleteNode = (record: any) => {
    const newConfig = cloneDeep(formConfigList);

    if (record.componentType === "Branch") {
      Modal.confirm({
        title: "删除字段",
        content: "删除该字段会删除所有子字段，是否继续?",
        okText: "删除",
        onOk() {
          const list = newConfig.filter((item) => item.key !== record.key);
          setFormConfigList(list);
        },
      });
      return;
    }
    /** 删除组组件成员 */
    if (record.groupId) {
      const group = newConfig.find((v) => v.key === record.groupId);
      group?.componentOption?.removeChild(record.key);
      setFormConfigList([...newConfig]);
      return;
    }
    /** 移除普通成员 */
    newConfig.splice(
      formConfigList.findIndex((item) => item.key === record.key),
      1
    );
    setFormConfigList(newConfig);
  };
  const columns: ColumnsType<BaseFormComponent> = [
    {
      title: "",
      dataIndex: "index",
      width: 40,
    },
    {
      title: "字段名称",
      dataIndex: "label",
    },
    {
      title: "类型",
      render(_, record) {
        const meta = record.meta();
        return meta.component.name;
      },
      key: "componentType",
    },

    {
      title: "提示",
      dataIndex: "componentOption",
      key: "name",
      render(_, record) {
        if (record.componentType === "Branch") {
          return (
            <Popover
              placement="top"
              trigger={"click"}
              open={popoverOpen[record.key]}
              content={
                <Menu
                  groupId={record.key}
                  onClick={() =>
                    setPopoverOpen({ ...popoverOpen, [record.key]: false })
                  }
                />
              }
              arrow={false}
              onOpenChange={() => {
                setPopoverOpen({ ...popoverOpen, [record.key]: true });
              }}
            >
              <Button type="primary" icon={<PlusCircleOutlined />} ghost>
                添加子字段
              </Button>
            </Popover>
          );
        }
        if (record.componentOption?.placeholder) {
          return record.componentOption.placeholder;
        }
        return "无";
      },
      onCell(record) {
        if (record.componentType === "Branch") {
          return {
            colSpan: 2,
          };
        }
        return {
          colSpan: 1,
        };
      },
    },
    {
      title: "必填",
      render(_, record) {
        return (
          <Checkbox value={record.componentOption?.require ?? false}></Checkbox>
        );
      },
      onCell(record) {
        if (record.componentType === "Branch") {
          return {
            colSpan: 0,
          };
        }
        return {
          colSpan: 1,
        };
      },
      key: "require",
    },

    {
      title: "操作",
      render(_, record) {
        return (
          <Space>
            <Button
              type="text"
              danger
              icon={<DeleteOutlined />}
              onClick={(e) => {
                e.stopPropagation();
                deleteNode(record);
              }}
            ></Button>
            {!record.groupId && (
              <Popover title="" content="拖动" trigger={"hover"}>
                <DragHandle />
              </Popover>
            )}
          </Space>
        );
      },
      key: "action",
    },
  ];

  const setRowClassName = (record: BaseFormComponent) => {
    if (record.groupId) {
      if (activeRow?.key === record.key) {
        return `${styles.rowGroup} ${styles.rowSelected}`;
      }
      return styles.rowGroup;
    }
    if (activeRow?.key === record.key) {
      return styles.rowSelected;
    }

    return "";
  };

  const onDragEnd = ({ active, over }: DragEndEvent) => {
    if (active.id !== over?.id) {
      setFormConfigList((prevState) => {
        const activeIndex = prevState.findIndex(
          (record) => record.key === active?.id
        );
        const overIndex = prevState.findIndex(
          (record) => record.key === over?.id
        );
        return arrayMove(prevState, activeIndex, overIndex);
      });
    }
  };

  return (
    <ConfigProvider
      theme={{
        components: {
          Table: {
            borderColor: "#8CB2FF",
            headerBg: "#F4F7FC",
            headerColor: "rgba(139, 146, 159, 1)",
            headerSplitColor: "transparent",
            rowSelectedBg: "#EDF5FF",
            rowHoverBg: "#edf5ff",
          },
        },
      }}
    >
      <DndContext modifiers={[restrictToVerticalAxis]} onDragEnd={onDragEnd}>
        <SortableContext
          items={formConfigList?.map((i) => i.key)}
          strategy={verticalListSortingStrategy}
        >
          <Table
            dataSource={TableData}
            columns={columns}
            components={{ body: { row: Row } }}
            pagination={false}
            scroll={{ y: 500 }}
            rowClassName={setRowClassName}
            onRow={onClickRow}
          />
        </SortableContext>
      </DndContext>
    </ConfigProvider>
  );
};

export default ComplexTable;
