import { Button, Form, Flex, Popover, message, notification } from "antd";
import styles from "./index.module.css";
import { useEffect, useMemo, useState } from "react";
import { PlusCircleOutlined } from "@ant-design/icons";
import Menu from "./menu";
import BaseFormComponent from "@/customForm/entity/base";
import ComplexTable from "./complex-table";
import RenderComponent from "../../../components/FormComponentRender/renderComponent";
import { PropInfo, PropInfoMap } from "@/customForm/interfaces";
import { useFormContext } from "..";
import service from "@/service";
import FormView from "./formView";
import eventBus from "@/utils/eventBus";
import { cloneDeep } from "lodash";

const FormRender = (data?: PropInfoMap<BaseFormComponent>) => {
  if (data) {
    const keys = Object.keys(data);

    return keys.map((key) => {
      const { config, desc, type } = data[
        key as keyof BaseFormComponent
      ] as PropInfo;
      if (!type) {
        return;
      }
      return (
        <Form.Item label={desc} name={key}>
          <RenderComponent
            config={config}
            componentType={type}
          ></RenderComponent>
        </Form.Item>
      );
    });
  }

  return null;
};

/**
 *
 * @returns
 */
const FormMatch = () => {
  /** 当前选中/操作的对象 */
  const [selectRow, setSelectRow] = useState<BaseFormComponent>();

  /** 当前操作的对象 */
  const actionObjMetadata = useMemo(
    () => selectRow?.meta() ?? undefined,
    [selectRow]
  );
  const [popoverOpen, setPopoverOpen] = useState(false);

  const {
    formConfigList,
    setFormConfigList,
    setActiveKey,
    setActiveTabs,
    baseInfo,
    mode,
  } = useFormContext();

  /** 当选中值配置映射被删除时，清掉右侧表单 */
  useEffect(() => {
    if (selectRow?.key) {
      /** 对子组件的处理 */
      if (selectRow?.groupId) {
        const group = formConfigList.find((v) => v.key === selectRow?.groupId);
        const has = group?.componentOption.children?.findIndex(
          (groupItem: any) => groupItem.key === selectRow?.key
        );

        if (has === -1) {
          setSelectRow(undefined);
        }
        return;
      }

      const has = formConfigList.findIndex((v) => {
        return v.key === selectRow?.key;
      });

      if (has === -1) {
        setSelectRow(undefined);
      }
    }
  }, [formConfigList, selectRow]);

  const name = actionObjMetadata?.component.name ?? "无";

  const [form] = Form.useForm();

  /** 更新选中 */
  useEffect(() => {
    const handler = (item: any) => {
      setSelectRow(item);
    };
    eventBus.on("add-form-item", handler);
    return () => {
      eventBus.off("add-form-item", handler);
    };
  }, []);

  useEffect(() => {
    if (!selectRow) {
      form.setFieldsValue({});
    }
    const meta = selectRow?.meta();
    /** 表单值初始化 */
    if (meta?.properties) {
      form.setFieldsValue({
        label: selectRow?.label,
        ...selectRow?.componentOption,
      });
    }
  }, [selectRow]);

  const onRowAddClickEvent = (record: BaseFormComponent) => {
    return {
      onClick: () => {
        if (selectRow?.key === record.key) {
          setSelectRow(undefined);
          return;
        }
        setSelectRow(record);
      },
    };
  };
  /** 更新原始值 */
  const updateOriginFormConfig = (values: Record<string, any>) => {
    /** 处置组值更改 */
    if (selectRow?.groupId) {
      const groupObj = formConfigList.find(
        (item) => item.key === selectRow?.groupId
      );

      const childrenList = cloneDeep(
        groupObj?.componentOption.children
      ) as BaseFormComponent[];

      const originObj = childrenList.find((v) => v.key === selectRow?.key);
      const { label, ...others } = values;

      originObj?.setValues({
        label: values.label,
        componentOption: originObj.componentOption?.update(others) ?? {},
      });

      if (groupObj) {
        groupObj.componentOption.children = childrenList;
      }
      setFormConfigList([...formConfigList]);

      return;
    }
    const originObj = formConfigList.find((v) => v.key === selectRow?.key);
    const { label, ...others } = values;

    originObj?.setValues({
      label: values.label,
      componentOption: originObj.componentOption?.update(others) ?? {},
    });

    /** 主动触发更新 */
    setFormConfigList([...formConfigList]);
  };

  const onFieldsChange = () => {
    form.validateFields().then((values) => {
      updateOriginFormConfig(values);
    });
  };

  /** 保存表单 */
  const submit = async () => {
    await service.managementService.removeAllConfigById(baseInfo?.id);
    const reqList = formConfigList.map((item, index) => {
      const newParams = {
        thingId: baseInfo?.id,
        label: item.label,
        componentType: item.componentType,
        fieldKey: item.key,
        validationRules: item.toJsonValidationRules(),
        otherSet: JSON.stringify(item.componentOption),
        groupId: item.groupId ?? "",
        sort: index + 1,
      };
      return service.settingService.saveFormConfigItem(newParams);
    });

    try {
      await Promise.all(reqList).then(() => {
        if (mode === "add") {
          setActiveTabs(["1", "2", "3"]);
          setActiveKey("3");
        } else {
          notification.success({
            message: "修改成功",
            duration: 1.5,
          });
        }
      });
    } catch (error) {
      message.error("保存失败");
    }
  };

  return (
    <div className={styles.container}>
      <div className={styles.cardWrapper}>
        <div className={styles.leftCard}>
          <ComplexTable
            activeRow={selectRow}
            onClickRow={onRowAddClickEvent}
          ></ComplexTable>
          <div className={styles.btnContent}>
            <Popover
              placement="top"
              trigger={"click"}
              content={<Menu onClick={() => setPopoverOpen(false)} />}
              arrow={false}
              open={popoverOpen}
              onOpenChange={() => {
                setPopoverOpen(!popoverOpen);
              }}
            >
              <Button type="primary" icon={<PlusCircleOutlined />} ghost>
                添加字段
              </Button>
            </Popover>
          </div>
        </div>
        <div className={styles.centerCard}>
          <Flex
            className={styles.titleWrapper}
            justify="space-between"
            align="center"
          >
            <div className={styles.rightTitle}>字段设置</div>
            <div>控件类型：{name}</div>
          </Flex>
          {selectRow ? (
            <div className={styles.formWrapper}>
              <Form
                form={form}
                colon={false}
                layout="vertical"
                onFieldsChange={onFieldsChange}
                wrapperCol={{ offset: 1, span: 22 }}
              >
                {FormRender(actionObjMetadata?.properties)}
              </Form>
            </div>
          ) : (
            <div className={styles.rightEmpty}>请先点击左侧表格控件</div>
          )}
        </div>
        <div className={styles.rightCard}>
          <Flex
            justify="space-between"
            align="center"
            className={styles.titleWrapper}
          >
            <div className={styles.rightTitle}>生成表单浏览</div>
          </Flex>
          <div className={styles.formWrapper}>
            <FormView />
          </div>
        </div>
      </div>
      <Button
        type="primary"
        onClick={submit}
        disabled={mode === "view"}
        style={{ float: "right", fontSize: 18, height: 50, marginTop: 16 }}
      >
        {mode === "add"
          ? "下一步"
          : mode === "view"
          ? "需无办理中申请才能编辑配置"
          : "保存"}
      </Button>
    </div>
  );
};

export default FormMatch;
